#ifndef _DB_OBJECT_MATCH_H_
#define _DB_OBJECT_MATCH_H_


#include "grtpp_util.h"
#include "wbpublic_public_interface.h"

#include "grt/common.h"
#include "grtdb/charset_utils.h"
#include "grtsqlparser/sql_facade.h"

namespace grt
{

struct WBPUBLICBACKEND_PUBLIC_FUNC DbObjectMatchAlterOmf : public Omf
{
  virtual bool less(const ValueRef& , const ValueRef&) const;
  virtual bool equal(const ValueRef& , const ValueRef&) const;
};

struct WBPUBLICBACKEND_PUBLIC_FUNC DbObjectMatchRecreateOmf : public Omf
{
  virtual bool less(const ValueRef& , const ValueRef&) const;
  virtual bool equal(const ValueRef& , const ValueRef&) const;
};

  template <typename Tgrt>
  class NormalizedComparer
    {
    protected:
      Tgrt _grt;
    public:
      NormalizedComparer(Tgrt grt):_grt(grt){};
      bool normalizedComparison(const ValueRef obj1, const ValueRef obj2, const std::string name)const
      {
        
        if((name == "comment"))
        {
          std::string sql1 = StringRef::cast_from(obj1);
          std::string sql2 = StringRef::cast_from(obj2);
          return bec::escape_sql_string(sql1) == bec::escape_sql_string(sql2);
        }
        
        std::string sql1= ObjectRef::cast_from(obj1).get_string_member(name);
        std::string sql2= ObjectRef::cast_from(obj2).get_string_member(name);
        SqlFacade* parser = SqlFacade::instance_for_rdbms_name(_grt, "Mysql");
        
        if (!parser)
          return false;
        
        sql1= parser->normalizeSqlStatement(sql1, GrtNamedObjectRef::cast_from(obj1)->owner()->name());
        sql2= parser->normalizeSqlStatement(sql2, GrtNamedObjectRef::cast_from(obj2)->owner()->name());
        sql1 = toupper(sql1);
        sql2 = toupper(sql2);
        return sql1 == sql2;
      };
      operator TSlotNormilizerSlot(){return sigc::mem_fun(*this, &NormalizedComparer<Tgrt>::normalizedComparison);};
    };
  
  
} // namespace grt

#endif // _DB_OBJECT_MATCH_H_
