#ifndef _SQL_NORMALIZER_H_
#define _SQL_NORMALIZER_H_


#include "wbpublic_public_interface.h"
#include <map>
#include <string>


#ifndef _WIN32
#include <string.h>
#define stricmp strcasecmp
#endif


/**
 * Defines interface to normalize provided SQL statement.
 * Normalization means transformation of all forms of equivalent SQL statements to the same basis form.
 * Addresses:
 * <li>optional braces
 * <li>optional default values
 * <li>forced quotation of identifiers
 * <li>etc.
 *
 * Used by SQL synchronization.
 * 
 * @ingroup sqlparser
 */
class WBPUBLICBACKEND_PUBLIC_FUNC Sql_normalizer
{
public:
  Sql_normalizer() : _delimiter("\\") {}
  virtual ~Sql_normalizer() {}

  void delimiter(std::string val) { _delimiter= val; }

  virtual std::string normalize(const std::string &sql, const std::string &schema_name)= 0;

protected:
  std::string _delimiter;
};


#endif // _SQL_NORMALIZER_H_
