#include "stdafx.h"

#include "wb_config.h"
#include "workbench_physical_model_impl.h"
#include "workbench_physical_connection_impl.h"

#include "grt/grt_manager.h"

#include "table_figure_wb.h"
#include "table_figure_idef1x.h"
#include "table_figure_simple.h"



workbench_physical_Model::ImplData::ImplData(workbench_physical_Model *self)
: super(self)
{
  _relationship_notation= PRCrowFoofnotation;
  _figure_notation= PFWorkbenchNotation;
  
  self->signal_changed().connect(sigc::mem_fun(this, &ImplData::member_changed_comm));
}


void workbench_physical_Model::ImplData::member_changed_comm(const std::string &name, const grt::ValueRef &ovalue)
{
  PhysicalRelationshipNotation rnot;
  PhysicalFigureNotation fnot;
  if (name == "connectionNotation")
  {
    std::string s= self()->_connectionNotation;
    if (s == "classic") rnot= PRClassicNotation;
    else if (s == "idef1x") rnot= PRIdef1xNotation;
    else if (s == "crowsfoot" || s == "ie") rnot= PRCrowFoofnotation;
    else if (s == "barker") rnot= PRBarkerNotation;
    else if (s == "uml") rnot= PRUMLNotation;
    else if (s == "fromcolumn") rnot= PRFromColumnNotation;
    else rnot= PRCrowFoofnotation;
    if (_relationship_notation != rnot)
    {
      _relationship_notation= rnot;
      run_later(sigc::mem_fun(this, &workbench_physical_Model::ImplData::reset_connections));
    }
  }
  else if (name == "figureNotation")
  {
    std::string s= self()->_figureNotation;
    if (s == "workbench" || s == "workbench/default") fnot= PFWorkbenchNotation;
    else if (s == "workbench/simple") fnot= PFWorkbenchSimpleNotation;
    else if (s == "workbench/pkonly") fnot= PFWorkbenchPKOnlyNotation;
    else if (s == "idef1x") fnot= PFIdef1xNotation;
    else if (s == "classic") fnot= PFClassicNotation;
    else if (s == "barker") fnot= PFBarkerNotation;
    else fnot= PFWorkbenchNotation;
    if (_figure_notation != fnot)
    {
      _figure_notation= fnot;
      run_later(sigc::mem_fun(this, &workbench_physical_Model::ImplData::reset_figures));
      run_later(sigc::mem_fun(this, &workbench_physical_Model::ImplData::reset_connections));  /* needed to restore the connection attachments */ 
    }
  }
}


mdc::LineEndType workbench_physical_Model::ImplData::get_line_end_type(bool mand, bool many, bool start)
{
  switch (_relationship_notation)
  {
  case PRIdef1xNotation:
    if (start)
      return mdc::FilledCircleEnd;
    else
      if (!mand) return mdc::HollowDiamondEnd; else return mdc::NormalEnd;
  case PRCrowFoofnotation:
    if (mand && many) return mdc::ChickenFoot1End;
    else if (!mand && many) return mdc::ChickenFoot0End;
    else if (mand && !many) return mdc::Cross1End;
    else return mdc::Cross0End;
  case PRBarkerNotation:
    if (many) return mdc::ChickenFootEnd;
    else return mdc::NormalEnd;
  case PRFromColumnNotation:
    return mdc::BoldStickEnd;
  case PRClassicNotation:
  case PRUMLNotation:
    return mdc::NormalEnd;
  }
  return mdc::NormalEnd;
}


std::string workbench_physical_Model::ImplData::get_line_end_caption(bool mand, bool many, bool start)
{
  switch (_relationship_notation)
  {
  case PRIdef1xNotation:
    if (start)
    {
      if (!mand && many) return "";
      else if (!mand && !many) return "Z";
      else if (mand && many)  return "P";
      else if (mand && !many) return "1";
    }
    break;

  case PRFromColumnNotation:
    if (start)
    {
      if (many) return "\xe2\x88\x9e";
      else return "1";
    }
    else
    {
      if (many) return "\xe2\x88\x9e";
      else return "1";
    }
    break;
  case PRCrowFoofnotation:
  case PRBarkerNotation:
    break;

  case PRClassicNotation:
  case PRUMLNotation:
    if (start)
    {
      if (!mand && many) return "0..*";
      else if (!mand && !many) return "0..1";
      else if (mand && many)  return "1..*";
      else if (mand && !many) return "1";
    }
    else
    {
      if (!mand && many) return "0..*";
      else if (!mand && !many) return "0..1";
      else if (mand && many) return "1..*";
      else if (mand && !many) return "1";
    }
    break;
  }
  return "";
}


void workbench_physical_Model::ImplData::update_relationship_figure(model_Connection::ImplData *cfig, bool imandatory, bool imany, bool fmandatory, bool fmany)
{
  wbfig::Connection *conn= dynamic_cast<wbfig::Connection*>(cfig->get_canvas_item());
  if (conn)
  {
    conn->set_end_type(get_line_end_type(imandatory, imany, true),
                       get_line_end_type(fmandatory, fmany, false));
    
    cfig->set_start_caption(get_line_end_caption(imandatory, imany, true));
    cfig->set_end_caption(get_line_end_caption(fmandatory, fmany, false));
    
    if (_relationship_notation == PRBarkerNotation)
    {
      conn->set_start_dashed(imandatory);
      conn->set_end_dashed(fmandatory);
    }
    else if (_relationship_notation == PRClassicNotation)
    {
      if (imany && fmany)
        conn->set_diamond_type(wbfig::Connection::Filled);
      else if (imany && !fmany)
        conn->set_diamond_type(wbfig::Connection::RightEmpty);
      else if (!imany && fmany)
        conn->set_diamond_type(wbfig::Connection::LeftEmpty);
      else
        conn->set_diamond_type(wbfig::Connection::Empty);
    }
    else if (_relationship_notation == PRFromColumnNotation)
    {
    }
    else
    {
      conn->set_start_dashed(false);
      conn->set_end_dashed(false);
    }
  }
}


wbfig::Table *workbench_physical_Model::ImplData::create_table_figure(mdc::Layer *layer)
{
  switch (_figure_notation)
  {
  case PFWorkbenchNotation:
    return new wbfig::WBTable(layer);    

  case PFWorkbenchSimpleNotation:
    {
      wbfig::WBTable *table= new wbfig::WBTable(layer);
      table->hide_indices();
      table->hide_triggers();
      return table;
    }

  case PFWorkbenchPKOnlyNotation:
    {
      wbfig::WBTable *table= new wbfig::WBTable(layer);
      table->hide_columns();
      table->hide_indices();
      table->hide_triggers();
      return table;
    }

  case PFIdef1xNotation:
    return new wbfig::Idef1xTable(layer);

  case PFClassicNotation:
    return new wbfig::SimpleTable(layer);

  case PFBarkerNotation:
  {
    wbfig::SimpleTable *table= new wbfig::SimpleTable(layer);
    table->set_barker_notation(true);
    return table;
  }
  default:
    return 0;
  }
}


static mdc::LineEndType get_line_end_type(bool mand, bool many, bool start)
{
  if (mand && many) return mdc::ChickenFoot1End;
  else if (!mand && many) return mdc::ChickenFoot0End;
  else if (mand && !many) return mdc::Cross1End;
  else return mdc::Cross0End;
}
