//
//  MCollectionViewItemView.m
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 12/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "MCollectionViewItemView.h"


@implementation MCollectionViewItemView
  
  
- (void)setOwner:(NSCollectionViewItem*)owner
{
  _owner= owner;
}


- (NSCollectionViewItem*)owner
{
  return _owner;
}


-(NSView *)hitTest:(NSPoint)aPoint 
{
  if (NSPointInRect(aPoint, [self convertRect:[self bounds] toView:[self superview]])) 
    return self;
  return nil;
}


-(void)mouseDown:(NSEvent *)theEvent 
{
  [super mouseDown:theEvent];

  // check for click count above one
  if([theEvent clickCount] > 1) 
  {
    if ([[self delegate] respondsToSelector:@selector(activateCollectionItem:)]) 
    {
      [[self delegate] activateCollectionItem: self];
    }
  }
}


- (void)setSelected:(BOOL)flag
{
  _selected= flag;
  [self setNeedsDisplay:YES];
  
  NSTextField *label= [self viewWithTag:1];
  if (!flag)
    [label setTextColor:[NSColor textColor]];
  else
    [label setTextColor:[NSColor alternateSelectedControlTextColor]];

  if ([[self delegate] respondsToSelector:@selector(selectCollectionItem:)]) 
    [[self delegate] selectCollectionItem:self];
}


- (BOOL)selected
{
  return _selected;
}


- (id)delegate
{
  return delegate;
}


- (void)setDelegate:(id)aDelegate
{
  delegate= aDelegate;
}


// sample code from quartz docs
static void addRoundedRectToPath(CGContextRef context, CGRect rect,
                                 float ovalWidth,float ovalHeight)

{
  float fw, fh;
  
  if (ovalWidth == 0 || ovalHeight == 0) {// 1
    CGContextAddRect(context, rect);
    return;
  }
  
  CGContextSaveGState(context);// 2
  
  CGContextTranslateCTM (context, CGRectGetMinX(rect),// 3
                         CGRectGetMinY(rect));
  CGContextScaleCTM (context, ovalWidth, ovalHeight);// 4
  fw = CGRectGetWidth (rect) / ovalWidth;// 5
  fh = CGRectGetHeight (rect) / ovalHeight;// 6
  
  CGContextMoveToPoint(context, fw, fh/2); // 7
  CGContextAddArcToPoint(context, fw, fh, fw/2, fh, 1);// 8
  CGContextAddArcToPoint(context, 0, fh, 0, fh/2, 1);// 9
  CGContextAddArcToPoint(context, 0, 0, fw/2, 0, 1);// 10
  CGContextAddArcToPoint(context, fw, 0, fw, fh/2, 1); // 11
  CGContextClosePath(context);// 12
  
  CGContextRestoreGState(context);// 13
}


- (void)drawRect:(NSRect)rect 
{
  if ([self selected])
  {
    CGContextRef context= [[NSGraphicsContext currentContext] graphicsPort];

    if ([[self window] firstResponder] == [self superview])
      [[NSColor alternateSelectedControlColor] setFill];
    else
      [[NSColor controlHighlightColor] setFill];
    
    addRoundedRectToPath(context, NSRectToCGRect(NSInsetRect([self bounds], 1.0, 1.0)), 6.0, 6.0);

    CGContextFillPath(context);
  }
  [super drawRect:rect];
}




@end

