//
//  ResponderLayer.m
//
//  Created by Jacob Engstrand on 2008-08-12.
//  This source code is public domain. No rights reserved.
//



#import "ResponderLayer.h"



@implementation ResponderLayer



- (ResponderLayer*) mouseDownAtPoint: (CGPoint) mouse;
{
	ResponderLayer* mouseHandledByLayer = nil;
	
	mMouseDownPoint = mouse;
	
	// Send mouse down message to all sublayers.
	NSArray* sublayers = [self sublayers];
	for (CALayer* lay in sublayers) {
		if ([lay respondsToSelector: @selector(mouseDownAtPoint:)]) {
			if (CGRectContainsPoint([lay frame], mouse)) {
				CGPoint localPoint = [self convertPoint: mouse
																				toLayer: lay];
				mouseHandledByLayer = [(ResponderLayer*)lay mouseDownAtPoint: localPoint];
			}
		}
		
		if (mouseHandledByLayer != nil) {
			break;
		}
	}
	
	return mouseHandledByLayer;
}



- (void) mouseDraggedToPoint: (CGPoint) mouse;
{
	if ([[self delegate] respondsToSelector: @selector(mouseDraggedToPoint:)]) {
		[[self delegate] mouseDraggedToPoint: mouse];
	}
}



- (void) mouseUp;
{
	if ([[self delegate] respondsToSelector: @selector(mouseUp)]) {
		[[self delegate] mouseUp];
	}
}



- (ResponderLayer*) responderLayerAtPoint: (CGPoint) mouse;
{
	ResponderLayer* layerAtPoint = nil;
	
	// Send mouse down message to all sublayers.
	NSArray* sublayers = [self sublayers];
	for (CALayer* lay in sublayers) {
		if ([lay respondsToSelector: @selector(responderLayerAtPoint:)]) {
			if (CGRectContainsPoint([lay frame], mouse)) {
				CGPoint localPoint = [self convertPoint: mouse
																				toLayer: lay];
				layerAtPoint = [(ResponderLayer*)lay responderLayerAtPoint: localPoint];
			}
		}
		
		if (layerAtPoint == nil) {
      // We return self since we are the last sublayer that responds to responderLayerAtPoint:.
			layerAtPoint = self;
		}
	}
	
	return layerAtPoint;
}



@end


