using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using Aga.Controls.Tree;
using MySQL.Grt;
using MySQL.Workbench;

namespace MySQL.GUI.Workbench
{
	public partial class ModelCatalogForm : DockContent
	{
		private WbContext wbContext;
		private GrtManager grtManager;
		private ModelCatalogTreeModel catalogTreeModel;
		private Point dragStart;

		private ModelCatalogForm()
		{
			InitializeComponent();
		}

		public ModelCatalogForm(WbContext WbContext)
			: this()
		{
			wbContext = WbContext;
			grtManager = wbContext.get_grt_manager();

      catalogTreeModel = new ModelCatalogTreeModel(catalogTreeView,
                    wbContext.get_catalog_tree(), nodeStateIcon);
      catalogTreeModel.AddColumn(nameNodeControl, 0, true);
      catalogTreeModel.AddColumn(presenceNodeControl, 1, false);
      catalogTreeView.Model = catalogTreeModel;

      UpdateCatalogTree();
		}

    public void RefreshContents()
    {
      catalogTreeView.Refresh();
    }

		public void UpdateCatalogTree()
		{
			if (catalogTreeModel != null)
				catalogTreeModel.RefreshModel();

      expandImportantNodes();
		}

    private void expandImportantNodes()
    {
      catalogTreeView.Root.Expand();
      foreach (TreeNodeAdv node in catalogTreeView.Root.Children)
      {
        node.Expand();

        foreach (TreeNodeAdv node2 in node.Children)
          node2.Expand();
      }
    }

		private void schemataTreeView_MouseDown(object sender, MouseEventArgs e)
		{
			dragStart = new Point(e.X, e.Y);
		}

		private void schemataTreeView_MouseMove(object sender, MouseEventArgs e)
		{
      if (e.Button == MouseButtons.Left && catalogTreeModel != null)
			{
				if (Math.Abs(dragStart.X - e.X) > 3 || Math.Abs(dragStart.Y - e.Y) > 3)
				{
          if (catalogTreeView.SelectedNodes.Count > 1)
          {
            List<GrtValue> list= new List<GrtValue>();
            foreach (TreeNodeAdv node in catalogTreeView.SelectedNodes)
            {
              GrtValue val = catalogTreeModel.GetNodeGrtValue(node);

              if (val != null)
                list.Add(val);
            }
            if (list.Count > 0)
              DoDragDrop(list, DragDropEffects.Copy);
          }
          else
          {
            GrtValue val = catalogTreeModel.GetNodeGrtValue(catalogTreeView.SelectedNode);

            if (val != null)
              DoDragDrop(val, DragDropEffects.Copy);
          }
				}
			}
		}

		private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
		{
			UpdateCatalogTree();
		}

    private void renameToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (catalogTreeView.SelectedNode != null)
      {
        nameNodeControl.BeginEdit();
      }
    }
	}
}