#ifndef _LF_UTILITIES_H_
#define _LF_UTILITIES_H_

#include <mforms/mforms.h>


#include "lf_mforms.h"

namespace mforms {
namespace gtk {

class UtilitiesImpl
{ 
  static int show_message(const std::string &title, const std::string &text,
                          const std::string &ok, const std::string &cancel,
                          const std::string &other)
  {
    Gtk::MessageDialog dlg("<b>"+title+"</b>\n"+text, true,
                           Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE, true);
    
    dlg.add_button(ok, 1);
    if (!cancel.empty())
      dlg.add_button(cancel, 0);
    if (!other.empty())
      dlg.add_button(other, -1);
    
    return dlg.run();
  }

  
  static int show_error(const std::string &title, const std::string &text,
                        const std::string &ok, const std::string &cancel,
                        const std::string &other)
  {
    Gtk::MessageDialog dlg("<b>"+title+"</b>\n"+text, true,
                           Gtk::MESSAGE_ERROR, Gtk::BUTTONS_NONE, true);
    
    dlg.add_button(ok, 1);
    if (!cancel.empty())
      dlg.add_button(cancel, 0);
    if (!other.empty())
      dlg.add_button(other, -1);
    
    return dlg.run();
  }

  static int show_warning(const std::string &title, const std::string &text,
                        const std::string &ok, const std::string &cancel,
                        const std::string &other)
  {
    Gtk::MessageDialog dlg("<b>"+title+"</b>\n"+text, true,
                           Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true);
    
    dlg.add_button(ok, 1);
    if (!cancel.empty())
      dlg.add_button(cancel, 0);
    if (!other.empty())
      dlg.add_button(other, -1);
    
    return dlg.run();
  }

  static void set_clipboard_text(const std::string &text)
  {
    Gtk::Clipboard::get()->set_text(text);
  }

public:
  static void init()
  {
    ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();

    f->_utilities_impl.show_message= &UtilitiesImpl::show_message;
    f->_utilities_impl.show_error= &UtilitiesImpl::show_error;
    f->_utilities_impl.show_warning= &UtilitiesImpl::show_warning;
    f->_utilities_impl.set_clipboard_text= &UtilitiesImpl::set_clipboard_text;
  }
};

};
};

#endif
