#ifndef _MFORMS_TEXTBOX_H_
#define _MFORMS_TEXTBOX_H_

#include <mforms/base.h>
#include <mforms/control.h>

#include <sigc++/sigc++.h>

namespace mforms {
  enum ScrollBars {
    NoScrollBar= 0,      // Don't show any scroll bar.
    HorizontalScrollBar, // Show only the horizontal scroll bar.
    VerticalScrollBar,   // Show only the vertical scroll bar.
    BothScrollBars       // Show both scroll bars.
  };

  class TextBox;

  struct TextBoxImplPtrs
  {
    bool (__stdcall *create)(TextBox *self, ScrollBars scroll_bars);
    void (__stdcall *set_bordered)(TextBox *self, bool flag);
    void (__stdcall *set_read_only)(TextBox *self, bool flag);
    void (__stdcall *set_text)(TextBox *self, const std::string &text);
    std::string (__stdcall *get_text)(TextBox *self);
    void (__stdcall *append_text)(TextBox *self, const std::string &text);
    void (__stdcall *set_padding)(TextBox *self, int pad);
  };

  class MFORMS_EXPORT TextBox : public Control
  {
  public:
    TextBox(ScrollBars scroll_bars);

    void set_bordered(bool flag);
    void set_read_only(bool flag);
    void set_value(const std::string &text);
    virtual std::string get_string_value();
    
    void append_text(const std::string &text);
    
    void set_padding(int pad);

    sigc::signal<void> signal_changed() { return _signal_changed; }

  public:
    void callback();

  protected:
    TextBoxImplPtrs *_textbox_impl;
    
    sigc::signal<void> _signal_changed;
  };
};

#endif
