#include "stdafx.h"

#ifndef _WIN32
#include <sys/types.h>
#include <sys/stat.h>
#endif

// This file contains common TUT test cases for SQL parser, lexer and splitter.

#define VERBOSE_OUTPUT 0

//----------------------------------------------------------------------------------------------------------------------

#include "test.h"
#include "myx_sql_parser_public_interface.h"

// only module_sql_parser_test

extern "C"
{
  CHARSET_INFO *get_charset_by_name(const char *cs_name, int flags);
}

TEST_MODULE(module_sql_parser_test, "SQL parser test suite");

//----------------------------------------------------------------------------------------------------------------------

static const char *test_function_1_input[] =
{
  "CREATE DATABASE IF NOT EXISTS `i-flow_dev` CHARACTER SET latin1 COLLATE latin1_swedish_ci;",
  "select * from a; select * from b; insert into c values (1,2,3); /* comment ; */; ;",
  "select * from a1; delimiter %%%\n delimiter $$$%%%\n\r select * from a2 $$$%%% select ",
  "select * from x; # comment ; \n\r select * from b",
  "/* comment ; ",
  "\" string ",
  "\' string ",
  "-- comment ; something ",
  "# comment ; something ",
  "select * from z; // blablabla ",
  "DELIMITER |\r\n CREATE1 |CREATE2|",
  "SELECT `тест`",
  "DELIMI",
  "DELIMITER",
  "DELIMITER ",
  "DELIMITER |\r\nx|",
  "DELIMITER //\r\nSELECTz1//SELECTz2",
  NULL
};

static const char *test_function_1_output[] =
{
  "CREATE DATABASE IF NOT EXISTS `i-flow_dev` CHARACTER SET latin1 COLLATE latin1_swedish_ci",

  "select * from a",
  " select * from b",
  " insert into c values (1,2,3)",
  " /* comment ; */",
  //" ",  // empty statments are never returned

  "select * from a1",
  " select * from a2 ",
  " select ",

  "select * from x",
  " # comment ; \n\r select * from b",

  "/* comment ; ",

  "\" string ",

  "\' string ",

  "-- comment ; something ",

  "# comment ; something ",

  "select * from z",
  " // blablabla ",

  " CREATE1 ",
  "CREATE2",

  "SELECT `тест`",

  "DELIMI",

  "DELIMITER",

  "DELIMITER ",

  "x",

  "SELECTz1",
  "SELECTz2"
};

static int test_function_1_output_index;
static bool test_function_1_success_flag;

int test_function_1_cb(const MyxStatementParser *splitter, const char *sql, void *user_data)
{
  test_function_1_success_flag &= (strcmp(sql, test_function_1_output[test_function_1_output_index++]) == 0);
  return 0;
}

TEST_FUNCTION(1)
{
  test_function_1_output_index= 0;
  test_function_1_success_flag= true;

  CHARSET_INFO *cs= get_charset_by_name("utf8_bin", MYF(0));

  for(int i= 0; test_function_1_input[i] != NULL; i++)
  {
    myx_process_sql_statements(test_function_1_input[i], cs, test_function_1_cb, reinterpret_cast<void *>(i), MYX_SPM_NORMAL_MODE);
    ensure("test 20", test_function_1_success_flag);
  }
}

static int test_function_2_counter;

int test_function_2_cb(const MyxStatementParser *splitter, const char *sql, void *user_data)
{
  test_function_2_counter++;
  return 0;
}

TEST_FUNCTION(2)
{
  CHARSET_INFO *cs= get_charset_by_name("utf8_bin", MYF(0));

  test_function_2_counter= 0;
  myx_process_sql_statements_from_file("data/db/menagerie-db/cr_event_tbl.sql", cs, test_function_2_cb, NULL, MYX_SPM_NORMAL_MODE);
  ensure("test 25 (1)", test_function_2_counter == 2);

  test_function_2_counter= 0;
  myx_process_sql_statements_from_file("data/db/menagerie-db/cr_pet_tbl.sql", cs, test_function_2_cb, NULL, MYX_SPM_NORMAL_MODE);
  ensure("test 25 (2)", test_function_2_counter == 2);

  test_function_2_counter= 0;
  myx_process_sql_statements_from_file("data/db/menagerie-db/ins_puff_rec.sql", cs, test_function_2_cb, NULL, MYX_SPM_NORMAL_MODE);
  ensure("test 25 (3)", test_function_2_counter == 1);

  test_function_2_counter= 0;
  myx_process_sql_statements_from_file("data/db/menagerie-db/load_pet_tbl.sql", cs, test_function_2_cb, NULL, MYX_SPM_NORMAL_MODE);
  ensure("test 25 (4)", test_function_2_counter == 2);

  test_function_2_counter= 0;
  myx_process_sql_statements_from_file("data/db/sakila-db/sakila-schema.sql", cs, test_function_2_cb, NULL, MYX_SPM_NORMAL_MODE);
  ensure("test 25 (5)", test_function_2_counter == 41);
}

int test_function_30_cb(const MyxStatementParser *splitter, const char *sql, void *user_data)
{
  return 0;
}

#ifdef __WIN__

// performance test
TEST_FUNCTION(3)
{
  const char *filename= "data/db/sakila-db/sakila-data.sql";

  CHARSET_INFO *cs= get_charset_by_name("utf8_bin", MYF(0));
  struct _stat statbuf;

  _stat(filename, &statbuf);

  test_time_point t1;
  myx_process_sql_statements_from_file(filename, cs, test_function_30_cb, NULL, MYX_SPM_NORMAL_MODE);

#if VERBOSE_OUTPUT
  test_time_point t2;

  float time_rate= ((float)1000.)/(t2 - t1).get_ticks();
  float size_per_sec= ((float)statbuf.st_size)*time_rate/1024/1024;
  std::cout << "Splitter performance test: " << std::endl 
    << "sakila-data.sql was processed in " << (t2 - t1) << " [" << size_per_sec << " MB/sec]" << std::endl;
#endif
}

#endif

END_TESTS;

//----------------------------------------------------------------------------------------------------------------------

